module bnpef_find
! Find non-stationary prediction-error filter (helix magic)
  use bnhconest
  use bnhelix
  use weight
  use laplacf
  use cgstep_mod
  use solver_reg_mod
  use dottest
  implicit none
  contains
  subroutine find_pef( yy, pef, niter, eps, resd, weight,np)
    integer,             intent( in)              :: niter,np(:)
    real,                intent( in)              :: eps	       ! epsilon
    real, dimension( :), pointer                  :: yy, resd, weight,wt
    type( bnfilter)                               :: pef
    integer                                       :: is, ip, ih, nr, nh
    real, dimension (:), pointer                  :: dd,ee, flt
!    real, dimension (:), allocatable             :: flt	  
    real, dimension(2) :: dot1,dot2
    nr = size(pef%flt)
    nh = size(pef%lag)
    write(0,*)"pef find 1",nr,nh
    allocate(flt(nr),dd( size( yy)))
    write(0,*)"pef find 2",size(flt,1),size(flt)/product(np),size(flt)/nh
    call bnhconest_init( yy, pef,nh)

    write(0,*)"pef find 3",np,nh
    dd=-yy
    call laplacf_init(np,nh)
    call weight_init(weight)
    write(0,*)"pef find 4",size(flt),size(dd)
    call solver_regu(m=flt,d=dd,Fop=bnhconest_lop,Aop=laplacf_lop,Wop=weight_lop,stepper=cgstep,nAop=nr,niter=niter,eps=eps,verb=.false.,resd=resd)
    write(0,*)"pef find 5"
    call cgstep_close()
    call laplacf_close()
    call bnhconest_close()
    do ip = 1, product(np)
      do ih = 1, nh
        pef%flt( ih,ip) = flt( (ip-1)*nh + ih)
      end do
    end do 
    deallocate( flt)
    deallocate(dd)
  end subroutine 
end module 
